/**
 *******************************************************************************
 * @file  adc.c
 * @brief A/D converter driver program
 * 
 * @note 
 * Copyright (C) 2015 Renesas Electronics Corporation 
 * Copyright (C) 2015 Renesas System Design Co., Ltd. 
 * 
 * @par
 *  This is a sample program. 
 *  Renesas Electronics assumes no responsibility for any losses incurred. 
 * 
 *******************************************************************************
 */

/*============================================================================*/
/* I N C L U D E                                                              */
/*============================================================================*/
#include "adc/adc.h"

/*============================================================================*/
/* D E F I N E                                                                */
/*============================================================================*/

/*===========================================================================*/
/* S T R U C T                                                               */
/*===========================================================================*/

/*============================================================================*/
/* V A R I A B L E                                                            */
/*============================================================================*/

/*============================================================================*/
/* E X T E R N                                                                */
/*============================================================================*/

/*============================================================================*/
/* S T A T I C   F U N C T I O N   P R O T O T Y P E                          */
/*============================================================================*/

/*============================================================================*/
/* P R O G R A M                                                              */
/*============================================================================*/

/**
 ******************************************************************************
  @brief  ADC mode init:SoftwareTrigger/select/single/1buffer
  @param  ch         -- Convert channels
  @retval ER_OK      -- Successful
  @retval ER_PARAM   -- Parameter error
 ******************************************************************************
*/
ER_RET adc_swtrg_init( uint8_t ch )
{
	uint32_t irq_num;
	
	if(ch > 0x7){return ER_PARAM;}
	
	/* Enable Interrupt */
	irq_num = (uint8_t)ADC_IRQn;
	NVIC_EnableIRQ((IRQn_Type)irq_num);
	
	/* ADC setup */
	RIN_ADC->ADM0 |= (1 << 15); // release software-reset
	RIN_ADC->ADM1 = (0x0
	              | (0 <<  0) // TRG      - Software trigger
	              | (0 <<  0) // TRGIN    - Don't care
	              | (1 <<  2) // MS       - Select mode
	              | (0 <<  3) // RPS      - Single mode
	              | (0 <<  4) // BS       - 1 buffer mode
	              | (0 <<  8) // TRGEN1-0 - Don't care
	              | (0 << 12) // EGA1-0   - Disable HW trigger
	                );
	RIN_ADC->ADM2 = ((1 << ch) << 0); // CHSEL  - enable AINch
	RIN_ADC->ADM3 = (0x0
	              | (0x00 << 24) // ADIL7-0  - Convert in a row
	              | (0x0B << 16) // ADCMP7-0 - 0x0B @RIN32M4
	              | (0x05 << 0)  // ADSMP7-0 - 0x05 @RIN32M4
	                );
	RIN_ADC->ADINT = (0X0
	               | (0  << 31) // INTS     - Pulse signal
	               | (0  << 16) // CSEEN    - Disable Outputof channel selecting error interrupt
	               | ((1 << ch) <<  0)  // INTEN7-0 - Enable interrupt of channel ch
	                 );
	return ER_OK;
}

/**
 ******************************************************************************
  @brief  ADC mode init:HardwareTrigger/auto/scan/single/1buffer
  @param  ch         -- Convert channels
  @retval ER_OK      -- Successful
  @retval ER_PARAM   -- Parameter error
 ******************************************************************************
*/
ER_RET adc_hwtrg_init( uint8_t ch_mux )
{
	uint32_t irq_num;
	if(ch_mux == 0 ){return ER_PARAM;}
	
	/* Enable Interrupt */
	irq_num = (uint8_t)ADC_IRQn;
	NVIC_EnableIRQ((IRQn_Type)irq_num);
	
	uint8_t intch;
	
	/* Enable interrupt with MAX channel number */
	if((ch_mux & 0x80) == 0x80){intch = 0x7;}
	else if((ch_mux & 0x40) == 0x40){intch = 0x6;}
	else if((ch_mux & 0x20) == 0x20){intch = 0x5;}
	else if((ch_mux & 0x10) == 0x10){intch = 0x4;}
	else if((ch_mux & 0x08) == 0x08){intch = 0x3;}
	else if((ch_mux & 0x04) == 0x04){intch = 0x2;}
	else if((ch_mux & 0x02) == 0x02){intch = 0x1;}
	else if((ch_mux & 0x01) == 0x01){intch = 0x0;}
	
	/* System setup */
	RIN_SYS->SYSPCMD = 0x000000A5;
	RIN_SYS->SYSPCMD = 0x00000001;
	RIN_SYS->SYSPCMD = 0x0000FFFE;
	RIN_SYS->SYSPCMD = 0x00000001;
	RIN_SYS->INTM0 = 0x00400002; // Rising eadge
	RIN_SYS->SYSPCMD = 0x00000000;
	/* ADC setup */
	RIN_ADC->ADM0 |= (1 << 15); // release software-reset
	RIN_ADC->ADM1 = (0x0
	              | (1 <<  0) // TRG      - Hardware trigger
	              | (0 <<  1) // TRGIN    - Auto mode
	              | (0 <<  2) // MS       - Scan mode
	              | (0 <<  3) // RPS      - Repeat mode
	              | (0 <<  4) // BS       - 1 buffer mode
	              | (0 <<  8) // TRGEN1-0 - External trigger
	              | (2 << 12) // EGA1-0   - Rising eadge
	                );
	RIN_ADC->ADM2 = (ch_mux << 0); // CHSEL  - enable AINch
	RIN_ADC->ADM3 = (0x0
	              | (0x00 << 24) // ADIL7-0  - Convert in a row
	              | (0x0B << 16) // ADCMP7-0 - 0x0B @RIN32M4
	              | (0x05 << 0)  // ADSMP7-0 - 0x05 @RIN32M4
	                );
	RIN_ADC->ADINT = (0X0
	               | (0 <<    31) // INTS     - Pulse signal
	               | (0 <<    16) // CSEEN    - Disable Outputof channel selecting error interrupt
	               | (1 << intch) // INTEN7-0 - Enable interrupt of channel ch
	                 );
	return ER_OK;
}

/**
 ******************************************************************************
  @brief  ADC convert start
  @param  none
  @retval none
 ******************************************************************************
*/
void adc_start( void )
{
	uint8_t waittime = 0;
	
	RIN_ADC->ADM0 |= (1 << 2); // powerdown mode -> normal mode
	for(waittime = 0; waittime < 0x64; ){waittime++;} // wait 1us
	RIN_ADC->ADM0 |= (1 << 0); // start
}
  
/**
 ******************************************************************************
  @brief  ADC convert stop
  @param  none
  @retval none
 ******************************************************************************
*/
void adc_stop( void )
{
	RIN_ADC->ADM0 &= ~(1 << 0);	// stop
	while( (RIN_ADC->ADM0 & 0x1) == 0x1
	     || (RIN_ADC->ADM0 & 0x2) == 0x2 ){} // waiting...
}

/**
 ******************************************************************************
  @brief  Read ADCR7-0
  @param  ch         -- ADC channel
  @param  data       -- Read data
  @retval ER_OK      -- Successful
  @retval ER_PARAM   -- Parameter error
 ******************************************************************************
*/
ER_RET adc_read( uint8_t ch, uint32_t* data )
{
	if(ch == 0 )       *data = RIN_ADC->ADCR0;
	else if(ch == 1 ) *data = RIN_ADC->ADCR1;
	else if(ch == 2 ) *data = RIN_ADC->ADCR2;
	else if(ch == 3 ) *data = RIN_ADC->ADCR3;
	else if(ch == 4 ) *data = RIN_ADC->ADCR4;
	else if(ch == 5 ) *data = RIN_ADC->ADCR5;
	else if(ch == 6 ) *data = RIN_ADC->ADCR6;
	else if(ch == 7 ) *data = RIN_ADC->ADCR7;
	else 	return ER_PARAM;
	
	return ER_OK;
}

/**
 ******************************************************************************
  @brief  ADC interrupt
  @param  none
  @retval none
 ******************************************************************************
*/
void ADC_IRQHandler(void)
{
	uint8_t intst;
	
	if( (RIN_ADC->ADSTS & 0x00010000) == 0x00010000){ // channel select error
		
		RIN_ADC->ADSTS |= 0x00010000; // clear status
	}
	else if( (RIN_ADC->ADSTS & 0xff) != 0x00){
		intst = 0xff & RIN_ADC->ADSTS;
		
		if((intst & 0x1 ) == 0x1){
			RIN_ADC->ADSTS |= (1 << 0); // clear status
		}
		if((intst & 0x2 ) == 0x2){
			RIN_ADC->ADSTS |= (1 << 1); // clear status
		}
		if((intst & 0x4 ) == 0x4){
			RIN_ADC->ADSTS |= (1 << 2); // clear status
		}
		if((intst & 0x8 ) == 0x8){
			RIN_ADC->ADSTS |= (1 << 3); // clear status
		}
		if((intst & 0x10 ) == 0x10){
			RIN_ADC->ADSTS |= (1 << 4); // clear status
		}
		if((intst & 0x20 ) == 0x20){
			RIN_ADC->ADSTS |= (1 << 5); // clear status
		}
		if((intst & 0x40 ) == 0x40){
			RIN_ADC->ADSTS |= (1 << 6); // clear status
		}
		if((intst & 0x80 ) == 0x80){
			RIN_ADC->ADSTS |= (1 << 7); // clear status
		}
	}
}
