/**
 *******************************************************************************
 * @file  taud.c
 * @brief 16-bit Timer (TAUD) driver program
 * 
 * @note 
 * Copyright (C) 2015 Renesas Electronics Corporation 
 * Copyright (C) 2015 Renesas System Design Co., Ltd. 
 * 
 * @par
 *  This is a sample program. 
 *  Renesas Electronics assumes no responsibility for any losses incurred. 
 * 
 *******************************************************************************
 */

/*============================================================================*/
/* I N C L U D E                                                              */
/*============================================================================*/
#include "taud/taud.h"

/*============================================================================*/
/* D E F I N E                                                                */
/*============================================================================*/
#define MAX_TAUD_CHANNEL	(15)			/* Max timer channel number */

#define SYS_TAUD_CHANNEL	(0)				/* System timer channel */
#define SYS_TAUD_DIVIDE	(3)					/* System timer freqency divided by 2^n */

#define MAX_TAUD_INTERVAL_TIME	(655UL)		/* Max interval time (us) */
#define MIN_TAUD_INTERVAL_TIME	(1UL)		/* Min interval time (us) */

#define MAX_TAUD_ONECNT_HW_TIME	(655UL)		/* Max one count time (us) */
#define MIN_TAUD_ONECNT_HW_TIME	(1UL)		/* Min one count time (us) */

/*===========================================================================*/
/* S T R U C T                                                               */
/*===========================================================================*/

/*============================================================================*/
/* V A R I A B L E                                                            */
/*============================================================================*/

/*============================================================================*/
/* E X T E R N                                                                */
/*============================================================================*/

/*============================================================================*/
/* S T A T I C   F U N C T I O N   P R O T O T Y P E                          */
/*============================================================================*/
//static ER_RET set_port_toutd( uint8_t ch );
//static ER_RET set_port_tind( uint8_t ch );

/*============================================================================*/
/* P R O G R A M                                                              */
/*============================================================================*/
/**
 ******************************************************************************
  @brief  Initialize interval timer
  @param  ch		: channel number
  @param  i_time	: interval time (us)
  @retval ER_OK		: No error
  @retval ER_PARAM	: Parameter error
 ******************************************************************************
*/
ER_RET taud_interval_init( uint8_t ch,uint16_t i_time )
{
	uint32_t i_count;						// counter value
	uint8_t irq_num;

	//=========================================
	// Check parameter s
	//=========================================
	if (ch > MAX_TAUD_CHANNEL) {
		return ER_PARAM;
	}

	if ((i_time < MIN_TAUD_INTERVAL_TIME) || (MAX_TAUD_INTERVAL_TIME < i_time)) {
		return ER_PARAM;
	}

	//=========================================
	// Set count clock
	//=========================================
	/* Set Prescaler */
	RIN_TAUD->TAUDTPS &= ~(0xf << (4*ch));		// [4*m+3:4*m] clear bits (CKm: PCLK/2^0 = 100MHz)
	/* Set Baud rate (CK3 only) */
//	RIN_TAUD->TAUDBRS = 0x00000000;				// [7:0] Baud rate for CK3

	//=========================================
	// Set count value
	//=========================================
	{
		uint32_t *cdr_reg;

		/* set base pointer */
		cdr_reg = (uint32_t*)&RIN_TAUD->TAUDCDR0;

		i_count = ( i_time*(1000 / 10) - 1 );
		*(cdr_reg + ch) = i_count;				// [15:0] countdown start value
	}
	//=========================================
	// Set Interval timer mode
	//=========================================
	{
		uint32_t *cmor_reg;
		uint32_t *cmur_reg;

		/* set base pointer */
		cmor_reg = (uint32_t*)&RIN_TAUD->TAUDCMOR0;
		cmur_reg = (uint32_t*)&RIN_TAUD->TAUDCMUR0;

		*(cmor_reg + ch) = ((ch << 14) |		// [15:14] Prescaler output      : Select CKm
						    ( 0 << 12) |		// [13:12] Count clock           : Prescaler
						    ( 0 << 11) |		// [11]    Synchronous operation : (Not used)
						    ( 0 <<  8) |		// [10:8]  Start trigger         : Software trigger
						    ( 0 <<  6) |		// [7:6]   Capture overflow      : (Not used)
						    ( 0 <<  1) |		// [4:1]   Mode                  : Interval timer
						    ( 0 <<  0));		// [0]     Aux                   : Does not output INTn when count starts
		*(cmur_reg + ch) = (( 0 <<  0));		// [1:0]   Detect TINm edge      : (Not used)
	}
	//=========================================
	// Set timer output signal
	//=========================================
	RIN_TAUD->TAUDTOE  |=  ( 1 << ch );			// [m] TOEm (channel output enable)  : Enable
	RIN_TAUD->TAUDTO   &= ~( 1 << ch );			// [m]  TOm (TOUTm pin output level) : Low level
	RIN_TAUD->TAUDTOM  &= ~( 1 << ch );			// [m] TOMm (channel output mode)    : Independent
	RIN_TAUD->TAUDTOC  &= ~( 1 << ch );			// [m] TOCm (channel output config)  : Performs toggle operation
	RIN_TAUD->TAUDTOL  &= ~( 1 << ch );			// [m] TOLm (channel output level)   : Active high

	//=========================================
	// Set reload data control
	//=========================================
	RIN_TAUD->TAUDRDE  &= ~( 1 << ch );			// [m] RDEm (reload data enable) : disable
	RIN_TAUD->TAUDRDM  &= ~( 1 << ch );			// [m] RDMm (reload data mode)   : disable for simultaneous rewriting

	//=========================================
	// Enable interrupt
	//=========================================
	if(ch < 5)irq_num = (uint8_t)TAUDI0_IRQn + ch; 
	else      irq_num = (uint8_t)INTPZ11_IRQn + ch - 5;
//	NVIC_SetPriority((IRQn_Type)irq_num, 1);
	NVIC_EnableIRQ((IRQn_Type)irq_num);

	//=========================================
	// Set TOUTn port (if you use)
	//=========================================
//	set_port_toutd(ch);

	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Initialize one count timer( delay count function controlled by hardware)
  @param  ch		: channel number
  @param  o_time	: one count time (ns)
  @param  trg		: timer count start trigger
  @retval ER_OK		: No error
  @retval ER_PARAM	: Parameter error
 ******************************************************************************
*/
ER_RET taud_onecount_hwtrg_init( uint8_t ch, uint16_t o_time, uint32_t trg )
{
	uint32_t o_count;			// one_count time
	uint8_t irq_num;

	//=========================================
	// Check parameter 
	//=========================================
	if (ch > MAX_TAUD_CHANNEL) {
		return ER_PARAM;
	}

	// a permissible range of one count time : 10ns~655350ns
	if ((o_time < MIN_TAUD_ONECNT_HW_TIME) || (MAX_TAUD_ONECNT_HW_TIME < o_time)) {
		return ER_PARAM;
	}

	//=========================================
	// Set count clock
	//=========================================
	/* Set Prescaler */
	RIN_TAUD->TAUDTPS &= ~( 0xf << (4*ch)) | ( 0x0 << (4*ch));
	/* Set Baud rate (CK3 only) */
//	RIN_TAUD->TAUDBRS = 0x00000000;				// [7:0] Baud rate for CK3

	/* Set CNTn loaded value */
	// count value = ((one_count time(ns))/(count clock cycle(ns)) - 1)
	// min                10ns       ,   100MHz(10ns)   ->   count value : 0x0001
	// Max count value : 0xFFFF(65,535)	                ->   655.35us = 655,350ns
	// Max 655us = 655,000ns, 100MHz(10ns)              ->   count value : 0xFFDC
	o_count = ((o_time*(1000/10)) - 1);

	switch(ch)
	{
		case 0:
			RIN_TAUD->TAUDCDR0 = o_count;		// ch0 countdown start value
			break;
		case 1:
			RIN_TAUD->TAUDCDR1 = o_count;		// ch1 countdown start value
			break;
		case 2:
			RIN_TAUD->TAUDCDR2 = o_count;		// ch2 countdown start value
			break;
		case 3:
			RIN_TAUD->TAUDCDR3 = o_count;		// ch3 countdown start value
			break;
		case 4:
			RIN_TAUD->TAUDCDR4 = o_count;		// ch4 countdown start value
			break;
		case 5:
			RIN_TAUD->TAUDCDR5 = o_count;		// ch5 countdown start value
			break;
		case 6:
			RIN_TAUD->TAUDCDR6 = o_count;		// ch6 countdown start value
			break;
		case 7:
			RIN_TAUD->TAUDCDR7 = o_count;		// ch7 countdown start value
			break;
		case 8:
			RIN_TAUD->TAUDCDR8 = o_count;		// ch8 countdown start value
			break;
		case 9:
			RIN_TAUD->TAUDCDR9 = o_count;		// ch9 countdown start value
			break;
		case 10:
			RIN_TAUD->TAUDCDR10 = o_count;		// ch10 countdown start value
			break;
		case 11:
			RIN_TAUD->TAUDCDR11 = o_count;		// ch11	countdown start value
			break;
		case 12:
			RIN_TAUD->TAUDCDR12 = o_count;		// ch12 countdown start value
			break;
		case 13:
			RIN_TAUD->TAUDCDR13 = o_count;		// ch13 countdown start value
			break;
		case 14:
			RIN_TAUD->TAUDCDR14 = o_count;		// ch14 countdown start value
			break;
		case 15:
			RIN_TAUD->TAUDCDR15 = o_count;		// ch15 countdown start value
			break;
		default:
			return ER_PARAM;
	}
	//=========================================
	// Set One count timer mode
	//=========================================
	{
		uint32_t *cmor_reg;
		uint32_t *cmur_reg;

		/* set base pointer */
		cmor_reg = (uint32_t*)&RIN_TAUD->TAUDCMOR0;
		cmur_reg = (uint32_t*)&RIN_TAUD->TAUDCMUR0;

		*(cmor_reg + ch) = ((ch << 14) |		// [15:14] Prescaler output      : Select CKm
							( 0 << 12) |		// [13:12] Count clock           : Prescaler
							( 0 << 11) |		// [11]    Synchronous operation : Slave
							( 1 <<  8) |		// [10:8]  Start trigger         : Software & TINn
							( 0 <<  6) |		// [7:6]   Capture overflow      : Not used,so set to 00B
							( 4 <<  1) |		// [4:1]   Mode                  : One count
							( 0 <<  0));		// [0]     Aux                   : Disable start trigger during count douwn
		*(cmur_reg + ch) = (( 1 <<  0));		// [1:0]   Detect TINm edge      : Rise edge
	}

	//=========================================
	// Set timer output signal
	//=========================================
	RIN_TAUD->TAUDTOE  &= ~( 1 << ch );			// [m] TOEm (channel output enable)  : Disable
	RIN_TAUD->TAUDTO   &= ~( 1 << ch );			// [m]  TOm (TOUTm pin output level) : Low level
	RIN_TAUD->TAUDTOM  &= ~( 1 << ch );			// [m] TOMm (channel output mode)    : Independent
	RIN_TAUD->TAUDTOC  &= ~( 1 << ch );			// [m] TOCm (channel output config)  : Performs toggle operation
	RIN_TAUD->TAUDTOL  &= ~( 1 << ch );			// [m] TOLm (channel output level)   : Active high

	//=========================================
	// Set reload data control
	//=========================================
	// Not used in this mode
	RIN_TAUD->TAUDRDE  &= ~( 1 << ch );			// [m] RDEm (reload data enable) : (Must be 0)
	RIN_TAUD->TAUDRDM  &= ~( 1 << ch );			// [m] RDMm (reload data mode)   : (Must be 0)

	//=========================================
	// Enable interrupt
	//=========================================
	if(ch < 5)irq_num = (uint8_t)TAUDI0_IRQn + ch; 
	else      irq_num = (uint8_t)INTPZ11_IRQn + ch - 5;
//	NVIC_SetPriority((IRQn_Type)irq_num, 1);
	NVIC_EnableIRQ((IRQn_Type)irq_num);

	//=========================================
	// Set TINm port (if you use)
	//=========================================
//	set_port_tind(ch);
	
	// Unlock system register
	RIN_SYS->SYSPCMD = 0x00a5;
	RIN_SYS->SYSPCMD = 0x0001;
	RIN_SYS->SYSPCMD = 0xfffe;
	RIN_SYS->SYSPCMD = 0x0001;

	// Use interrupt signal for timer trigger
	RIN_SYS->SELCNT |= (1 << (ch*2));
	switch(ch)
	{
		case 0:
			RIN_SYS->TMDTFR0  = trg;
			break;
		case 1:
			RIN_SYS->TMDTFR1  = trg;
			break;
		case 2:
			RIN_SYS->TMDTFR2  = trg;
			break;
		case 3:
			RIN_SYS->TMDTFR3  = trg;
			break;
		case 4:
			RIN_SYS->TMDTFR4  = trg;
			break;
		case 5:
			RIN_SYS->TMDTFR5  = trg;
			break;
		case 6:
			RIN_SYS->TMDTFR6  = trg;
			break;
		case 7:
			RIN_SYS->TMDTFR7  = trg;
			break;
		default:
			return ER_PARAM;
	}
	// Lock system register
	RIN_SYS->SYSPCMD = 0x0000;

	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Timer count start
  @param  ch		: channel number
  @retval ER_OK		: No error
  @retval ER_PARAM	: Parameter error
 ******************************************************************************
*/
ER_RET taud_start( uint8_t ch )
{
	/* Check parameter */
	if (ch > MAX_TAUD_CHANNEL) {
		return ER_PARAM;
	}

	/* Timer count start */
	RIN_TAUD->TAUDTS = (1 << ch);
	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Timer count stop
  @param  ch		: channel number
  @retval ER_OK		: No error
  @retval ER_PARAM	: Parameter error
 ******************************************************************************
*/
ER_RET taud_stop( uint8_t ch )
{
	/* Check parameter */
	if (ch > MAX_TAUD_CHANNEL) {
		return ER_PARAM;
	}

	/* Timer count stop */
	RIN_TAUD->TAUDTT = (1 << ch);

	return ER_OK;
}

/**
 ******************************************************************************
  @brief  Check TAUD activation
  @param  ch		: channel number
  @retval 0			: Not active
  @retval 1			: Active
 ******************************************************************************
*/

ER_RET taud_check_act(uint8_t ch)
{
	/* Check parameter */
	if (ch > MAX_TAUD_CHANNEL) {
		return ER_PARAM;
	}

	/* Read count operation status */
	if((RIN_TAUD->TAUDTE & (1 <<  ch)) == (1 << ch))
	{
		/* count operation is enabled */
		return 1;
	}
	else
	{
		/* count operation is disabled */
		return 0;
	}
}

/**
 ******************************************************************************
  @brief  Selcet TOUTm function port
  @param  ch		: channel number
  @retval ER_OK		: No error
  @retval ER_PARAM	: Parameter error
 ******************************************************************************
*/
#if 0
static ER_RET set_port_toutd( uint8_t ch )
{
	switch(ch)
	{
		case 0:
			RIN_EXTPORTB->EXTPFCE0B &= ~0x01;
			RIN_EXTPORTB->EXTPFC0B |= 0x01;
			RIN_EXTPORTB->EXTPMC0B |= 0x01;
			break;
		case 1:
			RIN_EXTPORTB->EXTPFCE0B &= ~0x02;
			RIN_EXTPORTB->EXTPFC0B |= 0x02;
			RIN_EXTPORTB->EXTPMC0B |= 0x02;
			break;
		case 2:
			RIN_EXTPORTB->EXTPFCE0B &= ~0x04;
			RIN_EXTPORTB->EXTPFC0B |= 0x04;
			RIN_EXTPORTB->EXTPMC0B |= 0x04;
			break;
		case 3:
			RIN_EXTPORTB->EXTPFCE0B &= ~0x08;
			RIN_EXTPORTB->EXTPFC0B |= 0x08;
			RIN_EXTPORTB->EXTPMC0B |= 0x08;
			break;
		case 4:
			RIN_GPIO->PFCE2B &= ~0x40;
			RIN_GPIO->PFC2B |= 0x40;
			RIN_GPIO->PMC2B |= 0x40;
			break;
		case 5:
			RIN_GPIO->PFCE2B &= ~0x80;
			RIN_GPIO->PFC2B |= 0x80;
			RIN_GPIO->PMC2B |= 0x80;
			break;
		case 6:
			RIN_GPIO->PFCE5B &= ~0x04;
			RIN_GPIO->PFC5B |= 0x04;
			RIN_GPIO->PMC5B |= 0x04;
			break;
		case 7:
			RIN_GPIO->PFCE5B &= ~0x80;
			RIN_GPIO->PFC5B |= 0x80;
			RIN_GPIO->PMC5B |= 0x80;
			break;
		case 8:
			RIN_RTPORT->RPFCE3B &= ~0x01;
			RIN_RTPORT->RPFC3B |= 0x01;
			RIN_RTPORT->RPMC3B |= 0x01;
			break;
		case 9:
			RIN_RTPORT->RPFCE3B &= ~0x02;
			RIN_RTPORT->RPFC3B |= 0x02;
			RIN_RTPORT->RPMC3B |= 0x02;
			break;
		case 10:
			RIN_RTPORT->RPFCE3B &= ~0x04;
			RIN_RTPORT->RPFC3B |= 0x04;
			RIN_RTPORT->RPMC3B |= 0x04;
			break;
		case 11:
			RIN_RTPORT->RPFCE3B &= ~0x08;
			RIN_RTPORT->RPFC3B |= 0x08;
			RIN_RTPORT->RPMC3B |= 0x08;
			break;
		case 12:
			RIN_RTPORT->RPFCE3B &= ~0x10;
			RIN_RTPORT->RPFC3B |= 0x10;
			RIN_RTPORT->RPMC3B |= 0x10;
			break;
		case 13:
			RIN_RTPORT->RPFCE3B &= ~0x2;
			RIN_RTPORT->RPFC3B |= 0x20;
			RIN_RTPORT->RPMC3B |= 0x20;
			break;
		case 14:
			RIN_RTPORT->RPFCE3B &= ~0x40;
			RIN_RTPORT->RPFC3B |= 0x40;
			RIN_RTPORT->RPMC3B |= 0x40;
			break;
		case 15:
			RIN_RTPORT->RPFCE3B &= ~0x80;
			RIN_RTPORT->RPFC3B |= 0x80;
			RIN_RTPORT->RPMC3B |= 0x80;
			break;
		default:
			return ER_PARAM;
			break;
	}
	return ER_OK;
}
#endif
/**
 ******************************************************************************
  @brief  Selcet TINm function port
  @param  ch		: channel number
  @retval ER_OK		: No error
  @retval ER_PARAM	: Parameter error
 ******************************************************************************
*/
#if 0
static ER_RET set_port_tind( uint8_t ch )
{
	switch(ch)
	{
		case 0:
			RIN_EXTPORTB->EXTPFCE0B |= 0x01;
			RIN_EXTPORTB->EXTPFC0B |= 0x01;
			RIN_EXTPORTB->EXTPMC0B |= 0x01;
			break;
		case 1:
			RIN_EXTPORTB->EXTPFCE0B |= 0x02;
			RIN_EXTPORTB->EXTPFC0B |= 0x02;
			RIN_EXTPORTB->EXTPMC0B |= 0x02;
			break;
		case 2:
			RIN_EXTPORTB->EXTPFCE0B |= 0x04;
			RIN_EXTPORTB->EXTPFC0B |= 0x04;
			RIN_EXTPORTB->EXTPMC0B |= 0x04;
			break;
		case 3:
			RIN_EXTPORTB->EXTPFCE0B |= 0x08;
			RIN_EXTPORTB->EXTPFC0B |= 0x08;
			RIN_EXTPORTB->EXTPMC0B |= 0x08;
			break;
		case 4:
			RIN_GPIO->PFCE2B &= ~0x40;
			RIN_GPIO->PFC2B &= ~0x40;
			RIN_GPIO->PMC2B |= 0x40;
			break;
		case 5:
			RIN_GPIO->PFCE2B &= ~0x80;
			RIN_GPIO->PFC2B &= ~0x80;
			RIN_GPIO->PMC2B |= 0x80;
			break;
		case 6:
			RIN_GPIO->PFCE5B &= ~0x04;
			RIN_GPIO->PFC5B &= ~0x04;
			RIN_GPIO->PMC5B |= 0x04;
			break;
		case 7:
			RIN_GPIO->PFCE5B &= ~0x80;
			RIN_GPIO->PFC5B &= ~0x80;
			RIN_GPIO->PMC5B |= 0x80;
			break;
		case 8:
			RIN_RTPORT->RPFCE3B |= 0x01;
			RIN_RTPORT->RPFC3B &= ~0x01;
			RIN_RTPORT->RPMC3B |= 0x01;
			break;
		case 9:
			RIN_RTPORT->RPFCE3B |= 0x02;
			RIN_RTPORT->RPFC3B &= ~0x02;
			RIN_RTPORT->RPMC3B |= 0x02;
			break;
		case 10:
			RIN_RTPORT->RPFCE3B |= 0x04;
			RIN_RTPORT->RPFC3B &= ~0x04;
			RIN_RTPORT->RPMC3B |= 0x04;
			break;
		case 11:
			RIN_RTPORT->RPFCE3B |= 0x08;
			RIN_RTPORT->RPFC3B &= ~0x08;
			RIN_RTPORT->RPMC3B |= 0x08;
			break;
		case 12:
			RIN_RTPORT->RPFCE3B |= 0x10;
			RIN_RTPORT->RPFC3B &= ~0x10;
			RIN_RTPORT->RPMC3B |= 0x10;
			break;
		case 13:
			RIN_RTPORT->RPFCE3B |= 0x2;
			RIN_RTPORT->RPFC3B &= ~0x20;
			RIN_RTPORT->RPMC3B |= 0x20;
			break;
		case 14:
			RIN_RTPORT->RPFCE3B |= 0x40;
			RIN_RTPORT->RPFC3B &= ~0x40;
			RIN_RTPORT->RPMC3B |= 0x40;
			break;
		case 15:
			RIN_RTPORT->RPFCE3B |= 0x80;
			RIN_RTPORT->RPFC3B &= ~0x80;
			RIN_RTPORT->RPMC3B |= 0x80;
			break;
		default:
			return ER_PARAM;
			break;
	}
	return ER_OK;
}
#endif

