/***********************************************************************/
/*                                                                     */
/*  FlashPrg.c:  Flash Programming Functions for                       */
/*               Renesas R-IN32M3 Device and                           */
/*               Spansion MX25L6433F Serial Flash                       */
/*                                                                     */
/***********************************************************************/

#include "..\FlashOS.H"        // FlashOS Structures
#include "RIN32M4.h"
#include "sromc.c"

/* commands */
#define SFLASH_COMMAND_PAGE_PROGRAM		(0x02)				/**< page program command */
#define SFLASH_COMMAND_READ_DATA		(0x03)				/**< read data command */
#define SFLASH_COMMAND_WRITE_DISABLE	(0x04)				/**< write disable command */
#define SFLASH_COMMAND_READ_STATUS		(0x05)				/**< read status register command */
#define SFLASH_COMMAND_WRITE_ENABLE		(0x06)				/**< write enable command */
#define SFLASH_COMMAND_ERASE_4K			(0x20)				/**< sector erase (4Kbyte) command */
#define SFLASH_COMMAND_ERASE_64K		(0xd8)				/**< sector erase (64Kbyte) command */
#define SFLASH_COMMAND_ERASE_CHIP		(0xc7)	
/* status register */
#define SFLASH_STATUS_BUSY				(1 << 0)			/**< erase/program in progress status */
#define SFLASH_STATUS_WEL				(1 << 1)			/**< write enable latch status */

static int sflash_wr_spi(uint8_t data, uint32_t first, uint32_t last);
static int sflash_rd_spi(uint8_t* data, uint32_t first, uint32_t last);
/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  /* disable watch dog timer in CC-Link IE module */
  (*((volatile uint16_t *) (0x40100962))) = 0x0000;

  sromc_init();

  return (0);                                  // Finished without Errors
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  return (0);                                  // Finished without Errors
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {
		uint8_t        status;
		/* write enable */
		sflash_wr_spi(SFLASH_COMMAND_WRITE_ENABLE, 1, 1);
		/* chip erase */
		sflash_wr_spi(SFLASH_COMMAND_ERASE_CHIP  , 1, 1);
		/* read status */
		do
		{
			sflash_wr_spi( SFLASH_COMMAND_READ_STATUS, 1, 0);
			sflash_rd_spi( &status                   , 0, 1);
		} while (status & SFLASH_STATUS_BUSY);	
  return (0);                                  
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {
			uint8_t        status;	
	    /* write enable */
	    sflash_wr_spi(SFLASH_COMMAND_WRITE_ENABLE, 1, 1);
			/* sector erase */
			sflash_wr_spi(SFLASH_COMMAND_ERASE_64K   , 1, 0);
			sflash_wr_spi( (adr >> 16) & 0xff        , 0, 0);
			sflash_wr_spi( (adr >>  8) & 0xff        , 0, 0);
			sflash_wr_spi( (adr >>  0) & 0xff        , 0, 1);
			/* read status */
			do
			{
				sflash_wr_spi(SFLASH_COMMAND_READ_STATUS, 1, 0);
				sflash_rd_spi(&status                   , 0, 1);
			} while (status & SFLASH_STATUS_BUSY);
			
  return (0);                                 

}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
	    uint32_t       i;
			uint8_t        status;	
	    /* write enable */
			sflash_wr_spi( SFLASH_COMMAND_WRITE_ENABLE, 1, 1);
			/* page program */
			sflash_wr_spi( SFLASH_COMMAND_PAGE_PROGRAM, 1, 0);
			sflash_wr_spi( (adr >> 16) & 0xff        , 0, 0);
			sflash_wr_spi( (adr >>  8) & 0xff        , 0, 0);
			sflash_wr_spi( (adr >>  0) & 0xff        , 0, 0);
	
			for (i = 0; i < (sz - 1); i++)
			{
				sflash_wr_spi( *buf++, 0, 0);
			}
			sflash_wr_spi( *buf++, 0, 1);
			/* read status */
			do
			{
				sflash_wr_spi(SFLASH_COMMAND_READ_STATUS, 1, 0);
				sflash_rd_spi( &status                   , 0, 1);
			} while (status & SFLASH_STATUS_BUSY);
					
  return (0);                                 
}
/**
 ******************************************************************************
  @brief  Write SPI
  @param  [in] data  -- Write data
  @param  [in] first -- flag for first SPI access cycle (0=NOT first, others=first)
  @param  [in] last  -- flag for last SPI access cycle (0=NOT last, others=last)
  @return Error condition
   0    -- Successful
   1    -- Parameter Error
 ******************************************************************************
 */
static int sflash_wr_spi(uint8_t data, uint32_t first, uint32_t last)
{
	sromc_write(data, first, last);
	return 0;
}
/**
 ******************************************************************************
  @brief  Read SPI
  @param  [out] *data -- Read data pointer
  @param  [in]  first -- Flag for first SPI access cycle (0=NOT first, others=first)
  @param  [in]  last  -- Flag for last SPI access cycle (0=NOT last, others=last)
  @return Error condition
  0    -- Successful
  1    -- Parameter Error
 ******************************************************************************
 */
static int sflash_rd_spi(uint8_t* data, uint32_t first, uint32_t last)
{
		sromc_read(data, first, last);
		return 0;
}
